@@ -17,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with gedit; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, 
+# Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA  02110-1301  USA
 
 import gedit
@@ -28,8 +28,8 @@ import gobject
 import vte
 import gconf
 import gettext
-import gnomevfs
 import os
+import gio
 from gpdefs import *
 
 try:
@@ -50,13 +50,14 @@ class GeditTerminal(gtk.HBox):
     }
 
     GCONF_PROFILE_DIR = "/apps/gnome-terminal/profiles/Default"
-    
+
     defaults = {
         'allow_bold'            : True,
         'audible_bell'          : False,
         'background'            : None,
         'backspace_binding'     : 'ascii-del',
-        'cursor_blinks'         : False,
+        'cursor_blink_mode'     : vte.CURSOR_BLINK_SYSTEM,
+        'cursor_shape'          : vte.CURSOR_SHAPE_BLOCK,
         'emulation'             : 'xterm',
         'font_name'             : 'Monospace 10',
         'scroll_on_keystroke'   : False,
@@ -78,11 +79,11 @@ class GeditTerminal(gtk.HBox):
         self._vte.set_size_request(200, 50)
         self._vte.show()
         self.pack_start(self._vte)
-        
+
         self._scrollbar = gtk.VScrollbar(self._vte.get_adjustment())
         self._scrollbar.show()
         self.pack_start(self._scrollbar, False, False, 0)
-        
+
         gconf_client.notify_add(self.GCONF_PROFILE_DIR,
                                 self.on_gconf_notification)
 
@@ -93,8 +94,24 @@ class GeditTerminal(gtk.HBox):
         self._vte.connect("popup-menu", self.on_vte_popup_menu)
         self._vte.connect("child-exited", lambda term: term.fork_command())
 
+        self._accel_base = '<gedit>/plugins/terminal'
+        self._accels = {
+            'copy-clipboard': [gtk.keysyms.C, gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, self.copy_clipboard],
+            'paste-clipboard': [gtk.keysyms.V, gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK, self.paste_clipboard]
+        }
+
+        for name in self._accels:
+            path = self._accel_base + '/' + name
+            accel = gtk.accel_map_lookup_entry(path)
+
+            if accel == None:
+                 gtk.accel_map_add_entry(path, self._accels[name][0], self._accels[name][1])
+
         self._vte.fork_command()
 
+    def do_grab_focus(self):
+        self._vte.grab_focus()
+
     def reconfigure_vte(self):
         # Fonts
         if gconf_get_bool(self.GCONF_PROFILE_DIR + "/use_system_font"):
@@ -114,6 +131,7 @@ class GeditTerminal(gtk.HBox):
         style = self._vte.get_style()
         fg = style.text[gtk.STATE_NORMAL]
         bg = style.base[gtk.STATE_NORMAL]
+        palette = []
 
         if not gconf_get_bool(self.GCONF_PROFILE_DIR + "/use_theme_colors"):
             fg_color = gconf_get_str(self.GCONF_PROFILE_DIR + "/foreground_color", None)
@@ -122,27 +140,48 @@ class GeditTerminal(gtk.HBox):
             bg_color = gconf_get_str(self.GCONF_PROFILE_DIR + "/background_color", None)
             if (bg_color):
                 bg = gtk.gdk.color_parse (bg_color)
-        self._vte.set_colors(fg, bg, [])
+        str_colors = gconf_get_str(self.GCONF_PROFILE_DIR + "/palette", None)
+        if (str_colors):
+            for str_color in str_colors.split(':'):
+                try:
+                    palette.append(gtk.gdk.color_parse(str_color))
+                except:
+                    palette = []
+                    break
+            if (len(palette) not in (0, 8, 16, 24)):
+                palette = []
+        self._vte.set_colors(fg, bg, palette)
 
         # cursor blink
         blink_mode = gconf_get_str(self.GCONF_PROFILE_DIR + "/cursor_blink_mode")
         if blink_mode.lower() == "system":
-            blink = gconf_get_bool("/desktop/gnome/interface/cursor_blink",
-                                   self.defaults['cursor_blinks'])
+            blink = vte.CURSOR_BLINK_SYSTEM
         elif blink_mode.lower() == "on":
-            blink = True
+            blink = vte.CURSOR_BLINK_ON
         elif blink_mode.lower() == "off":
-            blink = False
+            blink = vte.CURSOR_BLINK_OFF
         else:
-            blink = self.defaults['cursor_blinks']
-        self._vte.set_cursor_blinks(blink)
+            blink = self.defaults['cursor_blink_mode']
+        self._vte.set_cursor_blink_mode(blink)
+
+        # cursor shape
+        cursor_shape = gconf_get_str(self.GCONF_PROFILE_DIR + "/cursor_shape")
+        if cursor_shape.lower() == "block":
+            shape = vte.CURSOR_SHAPE_BLOCK
+        elif cursor_shape.lower() == "ibeam":
+            shape = vte.CURSOR_SHAPE_IBEAM
+        elif cursor_shape.lower() == "underline":
+            shape = vte.CURSOR_SHAPE_UNDERLINE
+        else:
+            shape = self.defaults['cursor_shape']
+        self._vte.set_cursor_shape(shape)
 
         self._vte.set_audible_bell(not gconf_get_bool(self.GCONF_PROFILE_DIR + "/silent_bell",
                                                       not self.defaults['audible_bell']))
 
         self._vte.set_scrollback_lines(gconf_get_int(self.GCONF_PROFILE_DIR + "/scrollback_lines",
                                                      self.defaults['scrollback_lines']))
-        
+
         self._vte.set_allow_bold(gconf_get_bool(self.GCONF_PROFILE_DIR + "/allow_bold",
                                                 self.defaults['allow_bold']))
 
@@ -170,10 +209,20 @@ class GeditTerminal(gtk.HBox):
             elif modifiers == gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK:
                 self.get_toplevel().child_focus(gtk.DIR_TAB_BACKWARD)
                 return True
+
+        for name in self._accels:
+            path = self._accel_base + '/' + name
+            entry = gtk.accel_map_lookup_entry(path)
+
+            if entry and entry[0] == event.keyval and entry[1] == modifiers:
+                self._accels[name][2]()
+                return True
+
         return False
 
     def on_vte_button_press(self, term, event):
         if event.button == 3:
+            self._vte.grab_focus()
             self.do_popup(event)
             return True
 
@@ -184,32 +233,43 @@ class GeditTerminal(gtk.HBox):
         menu = gtk.Menu()
 
         item = gtk.ImageMenuItem(gtk.STOCK_COPY)
-        item.connect("activate", lambda menu_item: self._vte.copy_clipboard())
+        item.connect("activate", lambda menu_item: self.copy_clipboard())
+        item.set_accel_path(self._accel_base + '/copy-clipboard')
         item.set_sensitive(self._vte.get_has_selection())
         menu.append(item)
 
         item = gtk.ImageMenuItem(gtk.STOCK_PASTE)
-        item.connect("activate", lambda menu_item: self._vte.paste_clipboard())
+        item.connect("activate", lambda menu_item: self.paste_clipboard())
+        item.set_accel_path(self._accel_base + '/paste-clipboard')
         menu.append(item)
-        
+
         self.emit("populate-popup", menu)
         menu.show_all()
         return menu
 
     def do_popup(self, event = None):
         menu = self.create_popup_menu()
-   
+
         if event is not None:
-	        menu.popup(None, None, None, event.button, event.time)
+            menu.popup(None, None, None, event.button, event.time)
         else:
             menu.popup(None, None,
                        lambda m: gedit.utils.menu_position_under_widget(m, self),
                        0, gtk.get_current_event_time())
-            menu.select_first(False)        
+            menu.select_first(False)
+
+    def copy_clipboard(self):
+        self._vte.copy_clipboard()
+        self._vte.grab_focus()
+
+    def paste_clipboard(self):
+        self._vte.paste_clipboard()
+        self._vte.grab_focus()
 
     def change_directory(self, path):
         path = path.replace('\\', '\\\\').replace('"', '\\"')
         self._vte.feed_child('cd "%s"\n' % path)
+        self._vte.grab_focus()
 
 class TerminalWindowHelper(object):
     def __init__(self, window):
@@ -228,7 +288,7 @@ class TerminalWindowHelper(object):
     def deactivate(self):
         bottom = self._window.get_bottom_panel()
         bottom.remove_item(self._panel)
-    
+
     def update_ui(self):
         pass
 
@@ -236,9 +296,10 @@ class TerminalWindowHelper(object):
         doc = self._window.get_active_document()
         if doc is None:
             return None
-        uri = doc.get_uri()
-        if uri is not None and gedit.utils.uri_has_file_scheme(uri):
-            return os.path.dirname(gnomevfs.get_local_path_from_uri(uri))
+        location = doc.get_location()
+        if location is not None and gedit.utils.uri_has_file_scheme(location.get_uri()):
+            directory = location.get_parent()
+            return directory.get_path()
         return None
 
     def on_panel_populate_popup(self, panel, menu):
@@ -269,7 +330,7 @@ class TerminalPlugin(gedit.Plugin):
 gconf_client = gconf.client_get_default()
 def gconf_get_bool(key, default = False):
     val = gconf_client.get(key)
-    
+
     if val is not None and val.type == gconf.VALUE_BOOL:
         return val.get_bool()
     else:
@@ -277,7 +338,7 @@ def gconf_get_bool(key, default = False):
 
 def gconf_get_str(key, default = ""):
     val = gconf_client.get(key)
-    
+
     if val is not None and val.type == gconf.VALUE_STRING:
         return val.get_string()
     else:
@@ -285,11 +346,11 @@ def gconf_get_str(key, default = ""):
 
 def gconf_get_int(key, default = 0):
     val = gconf_client.get(key)
-    
+
     if val is not None and val.type == gconf.VALUE_INT:
         return val.get_int()
     else:
         return default
 
-
-# ex:ts=4:et: Let's conform to PEP8
+# Let's conform to PEP8
+# ex:ts=4:et:
